#!/usr/bin/perl -w 
############################## check_snmp_fortigate_line_tracker ##############
# Version : 1.0.o
# Date : 17/03/2022
# Author  : Juergen Vigna (Wuerth Phoenix srl)
# Licence : GPL - http://www.fsf.org/licenses/gpl.txt
# TODO : 
##############################################################################
#
# Help : ./check_snmp_fortigate_line_tracker.pl -h
#

use strict;
use Net::SNMP;
use Getopt::Long;

# Nagios specific

use utils qw(%ERRORS $TIMEOUT);
#my $TIMEOUT = 15;
#my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# SNMP Datas

# Fortigate Line Tracker OID

my $baseoid = "1.3.6.1.4.1.12356.101.4.9.2.1";
my $locationoid  = "${baseoid}.2";
my $interfaceoid = "${baseoid}.14";
my $latencyoid   = "${baseoid}.5";
my $jitteroid    = "${baseoid}.6";
my $lossoid      = "${baseoid}.9";

# Globals

my $Version='1.0.0';

my $o_host = 	undef; 		# hostname
my $o_community = undef; 	# community
my $o_port = 	161; 		# port
my $o_help=	undef; 		# wan't some help ?
my $o_verb=	undef;		# verbose mode
my $o_version=	undef;		# print version
my $o_netsnmp=	1;		# Check with netsnmp (default)
my $o_cisco=	undef;		# Check cisco router mem
my $o_hp=	undef;		# Check hp procurve mem
my $o_warn=	undef;		# warning level option
my $o_warnL=	undef;		# warning level for latency
my $o_warnJ=	undef;		# warning level for jitter
my $o_warnO=	undef;		# warning level for loss
my $o_crit=	undef;		# Critical level option
my $o_critL=	undef;		# critical level for latency
my $o_critJ=	undef;		# critical level for jitter
my $o_critO=	undef;		# critical level for loss
my $o_perf=	undef;		# Performance data option
my $o_cache=	undef;		# Include cached memory as used memory
my $o_timeout=  undef; 		# Timeout (Default 5)
my $o_version2= undef;          # use snmp v2c
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

my $perfdata = "";
my $output = "";
my $outwarn = undef;
my $outcrit = undef;
my $state = 0;

# functions

sub p_version { print "check_snmp_fortigate_line_tracker version : $Version\n"; }

sub print_usage {
    print "Usage: $0 [-v] -H <host> -C <snmp_community> [-2] | (-l login -x passwd [-X pass -L <authp>,<privp>])  [-p <port>] -w <warn level> -c <crit level> [-f] [-t <timeout>] [-V]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub round ($$) {
    sprintf "%.$_[1]f", $_[0];
}

sub help {
   print "\nSNMP Fortigate Line Tracker Monitor for Nagios/Icinga version ",$Version,"\n";
   print "(c)2022 Author: Juergen Vigna\n\n";
   print_usage();
   print <<EOT;
-v, --verbose
   print extra debugging information (including interface list on the system)
-h, --help
   print this help message
-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies SNMP v1 or v2c with option)
-2, --v2c
   Use snmp v2c
-l, --login=LOGIN ; -x, --passwd=PASSWD
   Login and auth password for snmpv3 authentication 
   If no priv password exists, implies AuthNoPriv 
-X, --privpass=PASSWD
   Priv password for snmpv3 (AuthPriv protocol)
-L, --protocols=<authproto>,<privproto>
   <authproto> : Authentication protocol (md5|sha : default md5)
   <privproto> : Priv protocole (des|aes : default des) 
-P, --port=PORT
   SNMP port (Default 161)
-w, --warn=INT,INT,INT
   warning level for latency,jitter,loss
-c, --crit=INT,INT,INT
   critical level for latency,jitter,loss
-f, --perfdata
   Performance data output
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-V, --version
   prints version number
EOT
}

# For verbose output
sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

# Get the alarm signal (just in case snmp timout screws up)
$SIG{'ALRM'} = sub {
     print ("ERROR: Alarm signal (Nagios time-out)\n");
     exit $ERRORS{"UNKNOWN"};
};

sub check_options {
    Getopt::Long::Configure ("bundling");
	GetOptions(
   	'v'	=> \$o_verb,		'verbose'	=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
        'p:i'   => \$o_port,   		'port:i'	=> \$o_port,
        'C:s'   => \$o_community,	'community:s'	=> \$o_community,
	'l:s'	=> \$o_login,		'login:s'	=> \$o_login,
	'x:s'	=> \$o_passwd,		'passwd:s'	=> \$o_passwd,
	'X:s'	=> \$o_privpass,		'privpass:s'	=> \$o_privpass,
	'L:s'	=> \$v3protocols,		'protocols:s'	=> \$v3protocols,   
	't:i'   => \$o_timeout,       	'timeout:i'     => \$o_timeout,
	'V'	=> \$o_version,		'version'	=> \$o_version,
        '2'     => \$o_version2,        'v2c'           => \$o_version2,
        'c:s'   => \$o_crit,            'critical:s'    => \$o_crit,
        'w:s'   => \$o_warn,            'warn:s'        => \$o_warn,
        'f'     => \$o_perf,            'perfdata'      => \$o_perf
    );
    if (defined ($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
    if (defined($o_version)) { p_version(); exit $ERRORS{"UNKNOWN"}};
    if ( ! defined($o_host) ) # check host and filter 
	{ print "No host defined!\n";print_usage(); exit $ERRORS{"UNKNOWN"}}
    # check snmp information
    if ( !defined($o_community) && (!defined($o_login) || !defined($o_passwd)) )
	  { print "Put snmp login info!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if ((defined($o_login) || defined($o_passwd)) && (defined($o_community) || defined($o_version2)) )
	  { print "Can't mix snmp v1,2c,3 protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (defined ($v3protocols)) {
	  if (!defined($o_login)) { print "Put snmp V3 login info with protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	  my @v3proto=split(/,/,$v3protocols);
	  if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
	  if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
	  if ((defined ($v3proto[1])) && (!defined($o_privpass))) {
	    print "Put snmp V3 priv login info with priv protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	}
	if (defined($o_timeout) && (isnnum($o_timeout) || ($o_timeout < 2) || ($o_timeout > 60))) 
	  { print "Timeout must be >1 and <60 !\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (!defined($o_timeout)) {$o_timeout=5;}
	#Check Warning and crit are present
    if ( ! defined($o_warn) || ! defined($o_crit))
 	{ print "Put warning and critical values!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    # Get rid of % sign
    $o_warn =~ s/\%//g; 
    $o_crit =~ s/\%//g;
    my @o_warnL=split(/,/ , $o_warn);
    my @o_critL=split(/,/ , $o_crit);
    if ((@o_warnL != 3) || (@o_critL != 3)) {
        print "3 warnings and critical !\n";
        print_usage();
        exit $ERRORS{"UNKNOWN"}
    }
    for (my $i=0;$i<3;$i++) {
      if ( isnnum($o_warnL[$i]) || isnnum($o_critL[$i]))
          { print "Numeric value for warning or critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"} }
      if (($o_critL[$i]!= 0) && ($o_warnL[$i] > $o_critL[$i]))
         { print "warning <= critical ! \n";print_usage(); exit $ERRORS{"UNKNOWN"}}
    }
    $o_warnL=$o_warnL[0];$o_warnJ=$o_warnL[1];$o_warnO=$o_warnL[2];
    $o_critL=$o_critL[0];$o_critJ=$o_critL[1];$o_critO=$o_critL[2];
}

sub check_limits {
	my $name=shift;
	my $val=shift;
	my $warn=shift;
	my $crit=shift;
	my $st = 0;
	# GLOBAL: $outcrit, $outwarn, $state

	if ($val >= $crit) {
		$st = 2;
		if (defined($outcrit)) {
			$outcrit .= ", $name (CRITICAL: $val >= $crit)";
		} else {
			$outcrit = "$name (CRITICAL: $val >= $crit)";
		}
	}
	if ($st == 0 && $val >= $warn) {
		$st = 1;
		if (defined($outwarn)) {
			$outwarn .= ", $name (WARNING: $val >= $crit)";
		} else {
			$outwarn = "$name (WARNING: $val >= $crit)";
		}
	}
	if ($st > $state) {
		$state = $st;
	}
}

########## MAIN #######

check_options();

# Check gobal timeout if snmp screws up
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT");
  alarm($TIMEOUT);
} else {
  verb("no timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

# Connect to host
my ($session,$error);
if ( defined($o_login) && defined($o_passwd)) {
  # SNMPv3 login
  if (!defined ($o_privpass)) {
  verb("SNMPv3 AuthNoPriv login : $o_login, $o_authproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -timeout          => $o_timeout
    );  
  } else {
    verb("SNMPv3 AuthPriv login : $o_login, $o_authproto, $o_privproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -privpassword	=> $o_privpass,
	  -privprotocol => $o_privproto,
      -timeout      => $o_timeout
    );
  }
} else {
   if (defined ($o_version2)) {
     # SNMPv2 Login
	 verb("SNMP v2c login");
	 ($session, $error) = Net::SNMP->session(
	-hostname  => $o_host,
	    -version   => 2,
	-community => $o_community,
	-port      => $o_port,
	-timeout   => $o_timeout
     );
   } else {
    # SNMPV1 login
	verb("SNMP v1 login");
    ($session, $error) = Net::SNMP->session(
       -hostname  => $o_host,
       -community => $o_community,
       -port      => $o_port,
       -timeout   => $o_timeout
    );
  }
}
if (!defined($session)) {
	printf("ERROR opening session: %s.\n", $error);
	exit $ERRORS{"UNKNOWN"};
}

# Global variable
my $resultat;
my @res_location;
my @res_interface;
my @res_latency;
my @res_jitter;
my @res_loss;
my $n;
my @oid;

# Get Location Table
$resultat = (version->parse(Net::SNMP->VERSION) < 4) ?
               $session->get_table($locationoid)
               :$session->get_table(Baseoid => $locationoid);
if (!defined($resultat)) {
	printf("ERROR: Description table : %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
foreach my $key ( keys %$resultat) {
	verb("OID : $key, Desc : $$resultat{$key}");
	@oid=split (/\./,$key);
	$res_location[$oid[-1]] = $$resultat{$key};
}

# Get Interface-Name Table
$resultat = (version->parse(Net::SNMP->VERSION) < 4) ?
               $session->get_table($interfaceoid)
               :$session->get_table(Baseoid => $interfaceoid);
if (!defined($resultat)) {
	printf("ERROR: Description table : %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
foreach my $key ( keys %$resultat) {
	verb("OID : $key, Desc : $$resultat{$key}");
	@oid=split (/\./,$key);
	$res_interface[$oid[-1]] = $$resultat{$key};
}

# Get Latency Table
$resultat = (version->parse(Net::SNMP->VERSION) < 4) ?
               $session->get_table($latencyoid)
               :$session->get_table(Baseoid => $latencyoid);
if (!defined($resultat)) {
	printf("ERROR: Description table : %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
foreach my $key ( keys %$resultat) {
	verb("OID : $key, Desc : $$resultat{$key}");
	@oid=split (/\./,$key);
	$res_latency[$oid[-1]] = $$resultat{$key};
}

# Get Jitter Table
$resultat = (version->parse(Net::SNMP->VERSION) < 4) ?
               $session->get_table($jitteroid)
               :$session->get_table(Baseoid => $jitteroid);
if (!defined($resultat)) {
	printf("ERROR: Description table : %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
foreach my $key ( keys %$resultat) {
	verb("OID : $key, Desc : $$resultat{$key}");
	@oid=split (/\./,$key);
	$res_jitter[$oid[-1]] = $$resultat{$key};
}

# Get Loss Table
$resultat = (version->parse(Net::SNMP->VERSION) < 4) ?
               $session->get_table($lossoid)
               :$session->get_table(Baseoid => $lossoid);
if (!defined($resultat)) {
	printf("ERROR: Description table : %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
foreach my $key ( keys %$resultat) {
	verb("OID : $key, Desc : $$resultat{$key}");
	@oid=split (/\./,$key);
	$res_loss[$oid[-1]] = $$resultat{$key};
}

$n = 1;
while ($n < @res_location) {
	$perfdata="$perfdata " . $res_location[$n] . "-" . $res_interface[$n] . "-latency=" . $res_latency[$n] . ";$o_warnL;$o_critL;0;";
	check_limits($res_location[$n] . "-" . $res_interface[$n] . "-latency", $res_latency[$n],$o_warnL,$o_critL,$state);
	$perfdata="$perfdata " . $res_location[$n] . "-" . $res_interface[$n] . "-jitter=" . $res_jitter[$n] . ";$o_warnJ;$o_critJ;0;";
	check_limits($res_location[$n] . "-" . $res_interface[$n] . "-jitter", $res_jitter[$n],$o_warnJ,$o_critJ,$state);
	$perfdata="$perfdata " . $res_location[$n] . "-" . $res_interface[$n] . "-loss=" . $res_loss[$n] . "%;$o_warnO;$o_critO;0;";
	check_limits($res_location[$n] . "-" . $res_interface[$n] . "-loss", $res_loss[$n],$o_warnO,$o_critO,$state);
	$n = $n + 1;
}

if ($state == 2) {
	if (defined($o_perf)) {
		if (defined($outwarn)) {
			print "CRITICAL - $outcrit, $outwarn |$perfdata\n";
		} else {
			print "CRITICAL - $outcrit |$perfdata\n";
		}
	} else {
		if (defined($outwarn)) {
			print "CRITICAL - $outcrit, $outwarn\n";
		} else {
			print "CRITICAL - $outcrit\n";
		}
	}
} elsif ($state == 1) {
	if (defined($o_perf)) {
		print "WARNING - $outwarn |$perfdata\n";
	} else {
		print "WARNING - $outwarn\n";
	}
} else {
	if (defined($o_perf)) {
		print "OK - All Lines within limits |$perfdata\n";
	} else {
		print "OK - All Lines within limits\n";
	}
}
exit $state;
